/*
 * GNOME Contacts
 */

.contacts-map {
    background-color: @theme_bg_color;
}

/* The contacts in the left pane */
.contacts-contact-list {
  background-color: transparent;
}

/* A single row in the contact list pane */
row.contact-data-row {
}

/* Styles for a ContactsContactForm */
.contacts-contact-form {
  background-color: mix(@theme_bg_color, @theme_base_color, 0.4);
}

.contacts-suggestion {
  border-top: 1px solid @borders;
  background-color: shade(@theme_bg_color, 0.9);
}

/* Give the avatar in the ContactSheet some margin,
 * so it doesn't jump when switching to the editor. */
.contacts-contact-sheet .contacts-avatar {
  margin: 4px 8px;
}

.contacts-postal-entry {
 border-radius: 0 0 0 0;
 border-width: 1px 1px 0 1px;
}

.contacts-postal-entry:nth-child(first) {
 border-radius: 4px 4px 0 0;
}

.contacts-postal-entry:nth-child(last) {
 border-radius: 0 0 4px 4px;
 border-width: 1px;
}

/* The style for the background "watermark" image and text.
 * (copied from dim-label) */
.contacts-watermark {
  opacity: 0.55;
  text-shadow: none;
}

.contacts-avatar-popover .contact-display-name {
  font-size: 20px;
}

.avatar-button {
  border-radius: 50%;
  padding: 0 0;
}
